@@ -59,7 +59,7 @@ from .typeutils import (
 
 from .scalar import Int64
 from .array import Array
-
+from .context import Source
 
 log = logging.getLogger(__name__)
 
@@ -258,6 +258,12 @@ class MetaStruct(type):
                 _has_refs = True
                 break
         data['_has_refs'] = _has_refs
+        if '_extra_c_sources' not in data.keys():
+            data['_extra_c_sources'] = []
+        if '_depends_on' not in data.keys():
+            data['_depends_on'] = []
+        if '_kernels' not in data.keys():
+            data['_kernels'] = {}
 
         return type.__new__(cls, name, bases, data)
 
@@ -411,7 +417,10 @@ class Struct(metaclass=MetaStruct):
         from . import capi
 
         paths = cls._gen_data_paths()
-        return capi.gen_code(cls, paths, conf)
+
+        source = Source(source=capi.gen_code(cls, paths, conf),
+                        name=cls.__name__+'_gen_c_api')
+        return source
 
     @classmethod
     def _gen_c_decl(cls, conf=default_conf):
@@ -436,6 +445,40 @@ class Struct(metaclass=MetaStruct):
     def _get_inner_types(cls):
         return [fl.ftype for fl in cls._fields]
 
+    @property
+    def _context(self):
+        return self._buffer.context
+
+    @classmethod
+    def compile_class_kernels(cls, context, only_if_needed=False,
+                              apply_to_source=(),
+                              save_source_as=None,):
+
+        if only_if_needed:
+            all_found = True
+            for kk in cls._kernels.keys():
+                if kk not in context.kernels.keys():
+                    all_found = False
+                    break
+            if all_found:
+                return
+
+        context.add_kernels(
+            sources=[],
+            kernels=cls._kernels,
+            extra_classes=[cls],
+            apply_to_source=apply_to_source,
+            save_source_as=save_source_as,
+        )
+
+    def compile_kernels(self, only_if_needed=False,
+                        apply_to_source=(), save_source_as=None):
+        self.compile_class_kernels(
+            context=self._context, only_if_needed=only_if_needed,
+            apply_to_source=apply_to_source,
+            save_source_as=save_source_as)
+
+
 
 def is_struct(atype):
     return isinstance(atype, MetaStruct)
